//
// Function 1: Opening a VISA Session
//
#include <userint.h>
#include <formatio.h>
#include <utility.h>
#include <ansi_c.h>
#include <visa.h>
#include <math.h>

#define   PI   3.141592654

#define   INVALID_SESSION  -1
ViSession rmSession    = INVALID_SESSION;
ViSession instrSession = INVALID_SESSION;

/**
 * Open VISA connection.
 * This function opens a visa session.
 * \param sRscName = VISA resource string
 *		      GPIB::<n>::INSTR -> GPIB device on primary address n
 *		      TCPIP::<x>::gpib<b>,<d>::INSTR	-> TCP/IP converter, e.g. AD007
 *		      TCPIP::<type><serial>::INSTR  	-> TCPIP connection, e.g.
 *			rssmu200a100052, firewall needs to be deactivated
 */
ViBoolean arbDemoLibOpen (const char* sRscName)
{
   int iStatus = 0;
    
   // check if device is already open
   if (instrSession != INVALID_SESSION) 
      return VI_FALSE;
   // open default resource manager and instrument session 
   if (viOpenDefaultRM (&rmSession) < 0)
      return VI_FALSE;
   if ((iStatus=viOpen (rmSession, (char*)sRscName, VI_NULL, VI_NULL,
         &instrSession)) != VI_SUCCESS) {
      viClose (rmSession);
      instrSession = rmSession = INVALID_SESSION;
      return VI_FALSE;
   }
   // configure VISA formatted I/O
   do {
      // set timeout to 5000 ms, should be sufficient for most applications
      if ((iStatus = viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000)) < 0)
         break;
      // set input and output buffer sizes to 2 KByte
      if ((iStatus = viSetBuf (instrSession, VI_WRITE_BUF, 2048)) < 0) 
         break;
      if ((iStatus = viSetBuf (instrSession, VI_READ_BUF, 2048)) < 0) 
         break;
      if ((iStatus = viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE,
            VI_FLUSH_ON_ACCESS)) < 0) 
         break;
      if ((iStatus = viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE,
            VI_FLUSH_ON_ACCESS)) < 0) 
         break;
   } while (VI_FALSE);
   // clean up
   if (iStatus < 0)  {
      viClose (instrSession);
      viClose (rmSession);
      instrSession = rmSession = INVALID_SESSION;
      return VI_FALSE;
   }

   return VI_TRUE;
}

//
// Function 2: Closing a VISA Session
//

/**
 * Close VISA connection.
 * This function closes the VISA session.
 */
ViBoolean arbDemoLibClose (void)
{
  // check for valid session handle
  if (instrSession == INVALID_SESSION) 
    return VI_FALSE;
   // set instrument back to local mode
   viGpibControlREN(instrSession, VI_GPIB_REN_DEASSERT_GTL);

   if (instrSession != INVALID_SESSION) 	
      viClose (instrSession);
   if (rmSession != INVALID_SESSION) 		
      viClose (rmSession);
	
   // invalidate internal variables
   instrSession = INVALID_SESSION;
   rmSession = INVALID_SESSION;
	
   return VI_TRUE;
}

//
// Function 3: Sending a SCPI command to the instrument (write)
//

/**
 * Send command.
 * This function sends a command to the instrument. If the command is
 * terminated with *OPC? A read is automatically executed.
 * \param sOutBuf = output text string
 */
ViBoolean arbDemoLibWrite (const char *sOutBuf)
{
   int      iStatus = 0;
   ViUInt32 iRead   = 0;
   char	     cRecBuf[33];
	
   // check for valid session handle
   if (instrSession == INVALID_SESSION) 
      return VI_FALSE;
   // send data as text string
   if (strlen(sOutBuf)!=0) {
      if ((iStatus = viPrintf (instrSession, (char*)sOutBuf)) < 0)
         return VI_FALSE;
   }
   // terminated with *OPC?
   if (strstr(sOutBuf, "*OPC?")!=NULL) {
      memset (cRecBuf, 0, sizeof(cRecBuf));
      // receive OPC status information
      if ((iStatus = viRead (instrSession, (ViPBuf)cRecBuf, 
            sizeof(cRecBuf)-1, &iRead)) < 0)
         return VI_FALSE;
      cRecBuf[iRead]=0;
      // check result
      if (cRecBuf[0]!='1')
         return VI_FALSE;
   }
	
   return VI_TRUE;
}

//
// Function 4: Sending binary data block wise to the instrument
//

/**
 * Write binary data
 * This function sends binary data to the instrument.
 * \param cOutBuf = pointer to output buffer
 * \param iBufLen = length of output buffer
 */
ViBoolean arbDemoLibWriteBin(const char *cOutBuf, int iBufLen)
{
   int			iBytesRemain, iBytesToSend, iStatus = VI_TRUE;
   ViUInt32		iSentBytes;
   ViBoolean		bEOIStatus;
   unsigned int	iRead;				
	
   // check for valid session handle
   if (instrSession == INVALID_SESSION) 
      return VI_FALSE;
   // send data as binary
   if (iBufLen!=0 && cOutBuf!=NULL) {
      // use EOI line instead of character
      viGetAttribute(instrSession, VI_ATTR_SEND_END_EN, &bEOIStatus);
      viSetAttribute(instrSession, VI_ATTR_SEND_END_EN, VI_FALSE);
      iBytesRemain = iBufLen;
      while (iBytesRemain>0) {
         iBytesToSend = iBytesRemain;
         if (iBytesToSend>2048)
            iBytesToSend = 2048;
         // last transfer
         if (iBytesRemain<=iBytesToSend)
            viSetAttribute(instrSession, VI_ATTR_SEND_END_EN, VI_TRUE);
         // send out one block
         iStatus = viWrite (instrSession, (ViBuf)cOutBuf, iBytesToSend, &iSentBytes);
         if (iStatus < 0 || iBytesToSend!=iSentBytes) 
            return VI_FALSE;
         iBytesRemain -= iBytesToSend;
         cOutBuf += iSentBytes;
      }     
      // set EOI back to its old state
      viSetAttribute(instrSession, VI_ATTR_SEND_END_EN, bEOIStatus);
   }
    
   return VI_TRUE;
}

//
// The waveform data structure
//

/**
 * SMU wave file parameters
 * This structure is used with arbDemoLib_SMU_GenerateWvFile
 */
typedef struct
{
   unsigned int  iClockRate;	// clock rate in Hz
   unsigned int  iPoints;	// number of points
   double        dRmsOfs;	// RMS offset in dB
   double        dPeakOfs;	// Peak offset in dB
   char	          *szComment;	// comment
   double        *dI;		// buffer for I values
   double        *dQ;		// buffer for Q values
   unsigned char *ucFlags;	// buffer for control flags (NULL if unused)
} T_WV_DATA;

//
// Function 5: Generating a waveform file from existing I/Q data
//

/**
 * Create SMU waveform file from data.
 * This function creates a waveform file on the local hard drive.
 * \param tWvData = waveform data structure
 * \param szFile = file name of output file
 */
ViBoolean arbDemoLib_GenerateWvFile(T_WV_DATA *tWvData, const char *szFile)
{
   ViInt32	i,j;			  // counter variable
   ViInt16	_uiI, _uiQ;		  // IQ values as integer
   FILE		*fp;			  // file pointer
   ViInt32	_iYear, _iMonth, _iDay; // system date
   ViInt32	_iH, _iMin, _iSec;	  // system time
   ViChar	szTmp[256];
   ViInt16	iOld, iState;
	
   if (strlen(szFile)==0 || tWvData->iPoints == 0) 
      return VI_FALSE;
   // create waveform file
   // !!! must be in binary format !!!
   fp = fopen (szFile, "wb");
   if (fp==NULL) 
      return VI_FALSE;
   // write header and comment
   fprintf (fp, "{TYPE: SMU-WV,0}");
   fprintf (fp, "{COMMENT: %s}", tWvData->szComment);
   // write time stamp
   GetSystemDate (&_iMonth, &_iDay, &_iYear);
   GetSystemTime (&_iH, &_iMin, &_iSec);
   fprintf (fp, "{DATE: %04d-%02d-%02d;%02d:%02d:%02d}"
              ,_iYear, _iMonth, _iDay, _iH, _iMin, _iSec);
   fprintf (fp, "{CLOCK: %d}"          , tWvData->iClockRate);
   fprintf (fp, "{LEVEL OFFS: %f,%f}"  , tWvData->dRmsOfs, tWvData->dPeakOfs);
   fprintf (fp, "{SAMPLES: %d}"        , tWvData->iPoints);
   // write marker flags	
   if (tWvData->ucFlags != NULL) {
      // control list length
      fprintf (fp, "{CONTROL LENGTH: %d}", tWvData->iPoints);
      // write control block for markers 1...4, etc.
      for (i=0; i<4; i++) {
         fprintf (fp, "{MARKER LIST %d: ", i+1);
         iOld = -1;
         // all data points
         for (j=0; j<tWvData->iPoints; j++) {
            iState = tWvData->ucFlags[j] & (unsigned char)pow(2,i);
            if (iOld != iState) {
               if (iOld!=-1) fprintf (fp,";");
               fprintf (fp, "%d:%d", j, iState==0?0:1);
               iOld = iState;
            }
         }
         // terminate data section
         fprintf (fp, "}");
      }
   }
   // write waveform block
   fprintf (fp, "{WAVEFORM-%d: #", tWvData->iPoints*4+1) ;	// each pair is 4 bytes
   for (i=0; i<tWvData->iPoints; i++) {
      // _dI, _dQ values are +/-1.0
      _uiI = (ViInt16)(tWvData->dI[i]*32767.0+0.5);
      _uiQ = (ViInt16)(tWvData->dQ[i]*32767.0+0.5);
      fwrite (&_uiI,1,2,fp); fwrite (&_uiQ,1,2,fp);
   }
   fprintf (fp, "}");
   // close file
   fclose (fp);
			
   return VI_TRUE;
}

//
// Function 6: Uploading the waveform file to the instrument
//

/**
 * Upload a waveform file.
 * This function uploads a waveform file from the local hard drive
 * to the instrument. After the upload the file can be activated.
 * \param szFile = input file from local hard drive
 * \param szInstrFile = file on instrument
 * \param iBBPath = base band path 1 or 2
 * \param bActivate = activate after upload
 */
ViBoolean arbDemoLib_UploadWvFile(const char *szFile, const char *szInstrFile, 
				    int iBBPath, ViBoolean bActivate)
{
   FILE	            *fp;
   int	            iFileSize;	       // file size (input file)
   char	            *pcBuf,*pcBuf2;	// input file buffer
   int	            iLenHeader;
   char	            szTmp[256];
   unsigned short  iInterface;	// interface type
   char	            szBandPrefix[32];// band prefix for base band unit
   int             iNumRead;

   // check for valid session handle
   if (instrSession == INVALID_SESSION) return VI_FALSE;
   // setup BB prefix
   strcpy(szBandPrefix,"");
   if (iBBPath==1) strcpy(szBandPrefix,"SOUR1:");
   if (iBBPath==2) strcpy(szBandPrefix,"SOUR2:");
   if (strlen(szFile)==0 || strlen(szInstrFile)==0)
      return VI_FALSE;
   // get file size and allocate buffer
   GetFileSize (szFile, &iFileSize);
   if (iFileSize==0) 
      return VI_FALSE; 
   // set default directory for waveform file
   if (!arbDemoLibWrite (":MMEM:MSIS 'D:'; *OPC?")) 
      return VI_FALSE;
   if (!arbDemoLibWrite (":MMEM:CDIR '\\'; *OPC?")) 
      return VI_FALSE;
   // read waveform file, open as binary !
   fp = fopen (szFile, "rb");
   if (fp==NULL) 
      return VI_FALSE;
   // allocate large enough buffer and fill with zeros
   pcBuf = malloc (iFileSize + 256);
   if (pcBuf == NULL) { 
      fclose(fp); 
      return VI_FALSE; 
   }
   memset (pcBuf, 0, iFileSize+256);
   // create header of GPIB command
   sprintf (pcBuf, ":MMEM:DATA 'D:\\%s',#%d%d", 
      szInstrFile, (int)floor(log10(iFileSize)) + 1, iFileSize);
   iLenHeader = strlen(pcBuf);
   // read data to buffer and close file
   iNumRead = fread (&pcBuf[iLenHeader], 1, iFileSize, fp);
   fclose (fp);
   if (iNumRead != iFileSize) { 
      free(pcBuf); 
      return VI_FALSE; 
   }
   // append line feed
   sprintf (&pcBuf[iLenHeader+iFileSize], "\n");
   // get connection type and setup SMU to use EOI line
   viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &iInterface);
   if (iInterface == VI_INTF_GPIB)
      arbDemoLibWrite (":SYST:COMM:GPIB:LTER EOI; *OPC?");
   // transfer data
   if (!arbDemoLibWriteBin (pcBuf, iFileSize+iLenHeader+2))  { 
      free(pcBuf); 
      if (iInterface == VI_INTF_GPIB)
         arbDemoLibWrite (":SYST:COMM:GPIB:LTER STAN; *OPC?");
      return VI_FALSE; 
   } 
   free (pcBuf);
   // set SMU back to standard (line feed)
   if (iInterface == VI_INTF_GPIB)
      arbDemoLibWrite (":SYST:COMM:GPIB:LTER STAN; *OPC?");
   // activate wave form
   if (bActivate) {
      // preset ARB unit
      sprintf (szTmp, "%sBB:ARB:PRES; *OPC?", szBandPrefix);
      if (!arbDemoLibWrite (szTmp)) { return VI_FALSE; }
      // select waveform in generator
      sprintf (szTmp, "%sBB:ARB:WAV:SEL '%s'; *OPC?", szBandPrefix, szInstrFile);
      if (!arbDemoLibWrite (szTmp)) { return VI_FALSE; }
      // set state to ON
      sprintf (szTmp, "%sBB:ARB:STAT ON; *OPC?",szBandPrefix);
      if (!arbDemoLibWrite (szTmp)) { return VI_FALSE; }
   }
	
   return VI_TRUE;
}

//
// Main program: Using all of the above functions to generate a test signal
//

ViBoolean TestIt(void)
{
  int            i;
  double         dI[256], dQ[256];
  double         dRad;
  T_WV_DATA      WvData;
  unsigned char  ucFlags[256];

  // calculate waveform data, two sine waves
  for (i=0; i<256; i++)  {
    dRad  = 2.0*PI * (double)i / 256.0;
    dI[i] = 1.0 * sin(dRad + 0.0    ) + 0.0;
    dQ[i] = 1.0 * sin(dRad - PI/2.0 );
  }
  // setup flags
  memset (ucFlags, 0, 256);

  // fill waveform data structure
  WvData.szComment  = "Comment";
  WvData.iPoints    = 256;
  WvData.iClockRate = 54000000;
  WvData.dI         = dI;
  WvData.dQ         = dQ;
  WvData.dRmsOfs    = 0.0;
  WvData.dPeakOfs   = 0.0;
  WvData.ucFlags    = ucFlags;
  WvData.ucFlags    = ucFlags;

  // create file on local hard drive C:
  if (!arbDemoLib_GenerateWvFile (&WvData, "C:\\demo.wv"))
    return VI_FALSE;

  // open instrument connection
  if (!arbDemoLibOpen ("GPIB0::28::INSTR"))
    return VI_FALSE;

  // apply RF settings
  arbDemoLibWrite ("*RST");
  arbDemoLibWrite (":SOUR1:FREQ:MODE CW; *OPC?");
  arbDemoLibWrite (":SOUR1:FREQ 1 GHz; *OPC?");
  arbDemoLibWrite (":SOUR1:POW -30.0 dBm; *OPC?");
  arbDemoLibWrite (":OUTP1:STAT ON; *OPC?");

  // upload from C:\demo.wv -> d:\demo.wv on instrument
  if (!arbDemoLib_UploadWvFile("C:\\demo.wv", "D:\\demo.wv", 1, VI_TRUE))
    return VI_FALSE;

  if (!arbDemoLibClose())
    return VI_FALSE;

  return VI_TRUE;
}
 
/*
 * main program.
 */
int main (int argc, char *argv[])
{
  TestIt();

  MessagePopup ("Info","Done");

  return 0;
}




